import os, time, sys
from kodi_six import xbmc, xbmcaddon, xbmcgui, xbmcvfs

__py3__ = __py2__ = False
if sys.version_info[0] >= 3:	        __py3__ = True
else:				                    __py2__ = True

try:
    xbmc.translatePath = xbmcvfs.translatePath
except AttributeError:
    pass

__settings__     = xbmcaddon.Addon()
__pluginid__     = __settings__.getAddonInfo('id')
__plugin__       = __pluginid__.replace('plugin.video.','')
__addonpath__    = xbmc.translatePath( __settings__.getAddonInfo( 'path' ) )
__profilepath__  = xbmc.translatePath( __settings__.getAddonInfo('profile') )

__baseurl__      = __settings__.getLocalizedString(40000)
__baseloginurl__ = __settings__.getLocalizedString(40001)
__libname__      = __settings__.getLocalizedString(40002)
__rooturl__      = __settings__.getLocalizedString(40010)

__libpath__      = os.path.join( __addonpath__, "{0}.py".format(__libname__) )
__cookie__       = os.path.join( __profilepath__, "{0}.lwp".format(__plugin__) )

def log(msg):
    xbmc.log(msg="{0}: {1}".format(__plugin__.upper(), msg) , level=xbmc.LOGINFO)

try:
    __devicename__   = xbmc.getInfoLabel('System.FriendlyName')
except:
    __devicename__   = "Kodi (Generic)"

def remove(filename):
    if os.path.isfile(filename):
        os.remove(filename)

def parse_parameters():
    param       = []
    paramstring = sys.argv[2]
    if len( paramstring ) >= 2:
        params        = sys.argv[2]
        cleanedparams = params.replace( '?', '' )
        if ( params[len( params ) - 1] == '/' ):
            params = params[0:len( params ) - 2]
        pairsofparams = cleanedparams.split( '&' )
        param         = {}
        for i in range( len( pairsofparams ) ):
            splitparams = {}
            splitparams = pairsofparams[i].split( '=' )
            if ( len( splitparams ) ) == 2:
                param[splitparams[0]] = splitparams[1]

    return param

parameters = parse_parameters()

try:
    action = parameters["action"]
except:
    action = None
    pass

if action == "CLEAR_DATA":
    log("Clearing Cookie & Cache..")
    remove( __cookie__ )
    remove(os.path.join( __addonpath__, __libpath__ ))
    remove(os.path.join( __addonpath__, __libpath__ + "o" ))
    remove(os.path.join( __addonpath__, __libpath__ + "c" ))
    xbmc.executebuiltin('Notification(Success :-),'+__settings__.getLocalizedString(30523).replace('\\n','\n')+',3000,'+ os.path.join( __addonpath__, "resources", "media", "icon.png" ) +')')
    xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30523).replace('\\n','\n'))
    sys.exit(0)

def scrapCookie():
    with open(__cookie__, 'r') as content_file:
        page = content_file.read()
    import re
    amemberKey = re.search('amember_nr=(.+?);', page)
    if amemberKey:
        amemberKey = amemberKey.group(1)
        return amemberKey
    else:
        log("Unable to read memberid from cookie: {0}".format( __cookie__ ))
        log("Cookie Content: {0}".format( page ))
        sys.exit(1)

def getAddonLib(url):
    try:
        amemberid = scrapCookie()
        headers   = {"Referer": __devicename__, "Cookie":"amember_nr={0}".format( amemberid )}
        response  = requests.get(url, headers=headers, verify=False, timeout=30)
        data      = response.text
        return data
    except Exception:
        import traceback
        log(traceback.format_exc())
        dialog = xbmcgui.Dialog()
        dialog.ok(__plugin__.upper(), "Oops!","Failed to retrieve data from server","Please check internet on the device!")
        sys.exit(1)

def readMemberPage(url, username, password):
    data = ""
    try:
        millis   = str(int(round(time.time())))
        params   = {'amember_login': username, 'amember_pass': password, 'login_attempt_id': millis, 'amember_redirect_url': url}
        headers  = {"Referer": __devicename__}
        ses      = requests.Session()
        response = ses.post( __baseloginurl__, headers=headers, data=params, timeout=30 )
        data     = response.text
        with open(__cookie__, 'w') as f:
            for cookie in ses.cookies:
                f.write("{0}={1};".format( cookie.name, cookie.value ))
    except Exception:
        import traceback
        log(traceback.format_exc())
        dialog = xbmcgui.Dialog()
        dialog.ok(__plugin__.upper(), "Oops!","Failed to retrieve data from server","Please check internet on the device!")
        sys.exit(1)

    return data

def decode(string):
    key    = ''.join([chr(int(''.join(z), 16)) for z in zip('4D3067406D624F4B6875'[0::2],'4D3067406D624F4B6875'[1::2])])
    i      = 1
    result = ''
    for i in range(1,len(string)):
        char = string[i-1:i]
        if (i % len(key) == 0):
            keychar = key[len(key)-1:len(key)]
        else:
            keychar = key[(i % len(key) - 1):i % len(key)]

        char   = chr(ord(char) - ord(keychar))
        result = result + str(char)

    return result

def get_pycachefile():
    if __py3__:
        __pycachedir__   = os.path.join( __addonpath__, "__pycache__")
        _, files         = xbmcvfs.listdir( __pycachedir__ )
        for fileName in files:
            if fileName.startswith(__libname__) and fileName.endswith('.pyc'):
                return os.path.join( __pycachedir__, fileName )

    compiled_version_paths = ["{0}c".format(__libpath__),"{0}o".format(__libpath__)]
    for compiled_version_path in compiled_version_paths:
        if os.path.exists( compiled_version_path ):
            return compiled_version_path

    return None

def require_library_update():
    if os.path.exists( __libpath__ ):   remove( __libpath__ ) # Remove library if exists
    update_threshold = 3 * 3600 # 3 Hours
    __pycachefile__  = get_pycachefile()
    if __pycachefile__:
        if os.path.getsize(__pycachefile__) != 0 or (time.time() - os.path.getmtime(__pycachefile__)) < update_threshold:
            return False

    return True

def compileIt():
    exec( "import {0}\n".format( __libname__ ) )
    __pycachefile__ = get_pycachefile()
    if __pycachefile__ != "{0}c".format( __libpath__ ) or __pycachefile__ != "{0}o".format( __libpath__ ):
        if __py3__:
            os.replace( __pycachefile__, "{0}c".format( __libpath__ ) )
        # else:
            # try:
            #     os.remove( "{0}o".format( __libpath__ ) )
            # except: pass
            # os.rename(  __pycachefile__, "{0}o".format( __libpath__ ) )

status = 1
if require_library_update():
    status = 0
    log ("Validating " + __plugin__ + " credentials...")

    username = __settings__.getSetting('username')
    password = __settings__.getSetting('password')

    if not username or not password:
        log (__settings__.getLocalizedString(30501).replace('\\n','\n'))
        xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30501).replace('\\n','\n'))
        __settings__.openSettings()

    elif '@' in username:
        log (__settings__.getLocalizedString(30522).replace('\\n','\n'))
        xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30522).replace('\\n','\n'))
        __settings__.openSettings()

    else:
        import requests
        logindata = readMemberPage( __rooturl__, username, password )
        # log (logindata)
        if not logindata:
            log (__settings__.getLocalizedString(30504).replace('\\n','\n'))
            xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30504).replace('\\n','\n'))
            __settings__.openSettings()

        elif 'Please login' in logindata:
            log (__settings__.getLocalizedString(30502).replace('\\n','\n'))
            xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30502).replace('\\n','\n'))
            __settings__.openSettings()

        elif 'Access Denied' in logindata:
            log (__settings__.getLocalizedString(30521).replace('\\n','\n'))
            xbmcgui.Dialog().ok(__settings__.getLocalizedString(30500),__settings__.getLocalizedString(30521).replace('\\n','\n'))
            __settings__.openSettings()

        else:
            log ("Credentials are valid")
            url      = __baseurl__ + '/secured/xml/e-{0}3.py'.format(__libname__)
            encoded_data = getAddonLib( url )
            data         = decode( encoded_data )
            status       = 1
            with open( __libpath__, "w" ) as f:
                f.write(data)
            compileIt()

if status:
    exec( "import {0}\n{0}.main()".format( __libname__ ) )
